<?php

namespace Drupal\media_stream\Plugin\media\Source;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;

/**
 * Media source wrapping around a video stream.
 *
 * @MediaSource(
 *   id = "video_stream",
 *   label = @Translation("Video stream"),
 *   description = @Translation("Use video streams for reusable media."),
 *   allowed_field_types = {"uri"},
 *   default_thumbnail_filename = "video.png"
 * )
 */
class VideoStream extends MediaSourceBase {

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes(): array {
    return ['poster' => $this->t('Poster')];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    if ($attribute_name === 'poster') {
      return $this->getPoster($media);
    }
    return parent::getMetadata($media, $attribute_name);
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type): FieldConfigInterface {
    $storage = $this->getSourceFieldStorage() ?: $this->createSourceFieldStorage();
    return $this->entityTypeManager
      ->getStorage('field_config')
      ->create([
        'field_storage' => $storage,
        'bundle' => $type->id(),
        'label' => $this->t('@label URL', ['@label' => $this->pluginDefinition['label']]),
        'required' => TRUE,
      ]);
  }

  /**
   * {@inheritdoc}
   */
  public function prepareViewDisplay(MediaTypeInterface $type, EntityViewDisplayInterface $display): void {
    $display->setComponent($this->getSourceFieldDefinition($type)->getName(), [
      'type' => 'video_stream',
      'label' => 'visually_hidden',
    ]);
  }

  /**
   * Returns URL of the video poster.
   */
  protected function getPoster(MediaInterface $media): ?string {
    /** @var \Drupal\media\MediaTypeInterface $media_type */
    $media_type = $media->get('bundle')->entity;
    $field_name = $media_type->getFieldMap()['poster'] ?? NULL;
    if ($field_name && $media->hasField($field_name)) {
      /** @var \Drupal\file\Plugin\Field\FieldType\FileFieldItemList $image_field */
      $image_field = $media->get($field_name);
      if ($image_field->isEmpty() || !$image_field instanceof FieldItemListInterface) {
        return NULL;
      }
      return $image_field->referencedEntities()[0]->createFileUrl();
    }
    return NULL;
  }

}
